"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertEnumTypeNode = exports.isEnumTypeNode = exports.isDataEnum = exports.isScalarEnum = exports.enumTypeNodeFromIdl = exports.enumTypeNode = void 0;
const EnumEmptyVariantTypeNode_1 = require("./EnumEmptyVariantTypeNode");
const EnumStructVariantTypeNode_1 = require("./EnumStructVariantTypeNode");
const EnumTupleVariantTypeNode_1 = require("./EnumTupleVariantTypeNode");
function enumTypeNode(variants) {
    return { kind: 'enumTypeNode', variants };
}
exports.enumTypeNode = enumTypeNode;
function enumTypeNodeFromIdl(idl) {
    const variants = idl.variants.map((variant) => {
        if (!variant.fields || variant.fields.length <= 0) {
            return (0, EnumEmptyVariantTypeNode_1.enumEmptyVariantTypeNodeFromIdl)(variant);
        }
        if (isStructField(variant.fields[0])) {
            return (0, EnumStructVariantTypeNode_1.enumStructVariantTypeNodeFromIdl)(variant);
        }
        return (0, EnumTupleVariantTypeNode_1.enumTupleVariantTypeNodeFromIdl)(variant);
    });
    return enumTypeNode(variants);
}
exports.enumTypeNodeFromIdl = enumTypeNodeFromIdl;
function isScalarEnum(node) {
    return node.variants.every((variant) => variant.kind === 'enumEmptyVariantTypeNode');
}
exports.isScalarEnum = isScalarEnum;
function isDataEnum(node) {
    return !isScalarEnum(node);
}
exports.isDataEnum = isDataEnum;
function isEnumTypeNode(node) {
    return !!node && node.kind === 'enumTypeNode';
}
exports.isEnumTypeNode = isEnumTypeNode;
function assertEnumTypeNode(node) {
    var _a;
    if (!isEnumTypeNode(node)) {
        throw new Error(`Expected enumTypeNode, got ${(_a = node === null || node === void 0 ? void 0 : node.kind) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertEnumTypeNode = assertEnumTypeNode;
function isStructField(field) {
    return typeof field === 'object' && 'name' in field && 'type' in field;
}
//# sourceMappingURL=EnumTypeNode.js.map