"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutoSetAnchorDiscriminatorsVisitor = void 0;
const sha256_1 = require("@noble/hashes/sha256");
const nodes = __importStar(require("../../nodes"));
const shared_1 = require("../../shared");
const BaseNodeVisitor_1 = require("../BaseNodeVisitor");
const Visitor_1 = require("../Visitor");
class AutoSetAnchorDiscriminatorsVisitor extends BaseNodeVisitor_1.BaseNodeVisitor {
    constructor() {
        super(...arguments);
        this.program = null;
    }
    visitProgram(program) {
        this.program = program;
        const visitedProgram = nodes.programNode({
            ...program,
            accounts: program.accounts
                .map((account) => (0, Visitor_1.visit)(account, this))
                .filter(nodes.assertNodeFilter(nodes.assertAccountNode)),
            instructions: program.instructions
                .map((instruction) => (0, Visitor_1.visit)(instruction, this))
                .filter(nodes.assertNodeFilter(nodes.assertInstructionNode)),
        });
        this.program = null;
        return visitedProgram;
    }
    visitAccount(account) {
        var _a;
        const shouldAddDiscriminator = ((_a = this.program) === null || _a === void 0 ? void 0 : _a.origin) === 'anchor';
        if (!shouldAddDiscriminator)
            return account;
        const idlName = (0, shared_1.snakeCase)(account.idlName);
        const hash = (0, sha256_1.sha256)(`global:${idlName}`).slice(0, 8);
        const discriminatorField = nodes.structFieldTypeNode({
            name: 'discriminator',
            child: nodes.arrayTypeNode(nodes.numberTypeNode('u8'), {
                size: (0, shared_1.fixedSize)(8),
            }),
            defaultsTo: {
                strategy: 'omitted',
                value: getValueNodeFromBytes(hash),
            },
        });
        return nodes.accountNode({
            ...account,
            discriminator: (0, shared_1.fieldAccountDiscriminator)('discriminator'),
            data: nodes.accountDataNode({
                ...account.data,
                struct: nodes.structTypeNode([
                    discriminatorField,
                    ...account.data.struct.fields,
                ]),
            }),
        });
    }
    visitInstruction(instruction) {
        var _a;
        const shouldAddDiscriminator = ((_a = this.program) === null || _a === void 0 ? void 0 : _a.origin) === 'anchor';
        if (!shouldAddDiscriminator)
            return instruction;
        const idlName = (0, shared_1.snakeCase)(instruction.idlName);
        const hash = (0, sha256_1.sha256)(`global:${idlName}`).slice(0, 8);
        const discriminatorField = nodes.structFieldTypeNode({
            name: 'discriminator',
            child: nodes.arrayTypeNode(nodes.numberTypeNode('u8'), {
                size: (0, shared_1.fixedSize)(8),
            }),
            defaultsTo: {
                strategy: 'omitted',
                value: getValueNodeFromBytes(hash),
            },
        });
        return nodes.instructionNode({
            ...instruction,
            dataArgs: nodes.instructionDataArgsNode({
                ...instruction.dataArgs,
                struct: nodes.structTypeNode([
                    discriminatorField,
                    ...instruction.dataArgs.struct.fields,
                ]),
            }),
        });
    }
}
exports.AutoSetAnchorDiscriminatorsVisitor = AutoSetAnchorDiscriminatorsVisitor;
function getValueNodeFromBytes(bytes) {
    return nodes.vList([...bytes].map((byte) => nodes.vScalar(byte)));
}
//# sourceMappingURL=AutoSetAnchorDiscriminatorsVisitor.js.map